from __future__ import annotations

from . import style


class ColorEscapeSequences(str):
    
    def __new__(cls, code: int) -> ColorEscapeSequences:
        obj = super().__new__(cls, ColorEscapeSequences.to_escseq(code))
        return obj


    def __init__(self, code: int) -> None:
        self.fg = ColorEscapeSequences.to_escseq(code)
        self.bg = ColorEscapeSequences.to_escseq(code + 10)


    def __str__(self) -> str:
        return self.fg
    
    
    def __repr__(self) -> str:
        return 'ColorEscapeSequences(fg=%r, bg=%r)' % (self.fg, self.bg)


    def __add__(self, other: str | ColorEscapeSequences) -> str:
        return self.fg + other
    
    
    @staticmethod
    def to_escseq(code: int) -> str:
        return f'\x1b[{code}m'


DEFAULT_COLOR = ColorEscapeSequences(39)

BLACK = ColorEscapeSequences(30)
RED = ColorEscapeSequences(31)
GREEN = ColorEscapeSequences(32)
YELLOW = ColorEscapeSequences(33)
BLUE = ColorEscapeSequences(34)
MAGENTA = ColorEscapeSequences(35)
CYAN = ColorEscapeSequences(36)
WHITE = ColorEscapeSequences(37)

BBLACK = ColorEscapeSequences(90)
BRED = ColorEscapeSequences(91)
BGREEN = ColorEscapeSequences(92)
BYELLOW = ColorEscapeSequences(93)
BBLUE = ColorEscapeSequences(94)
BMAGENTA = ColorEscapeSequences(95)
BCYAN = ColorEscapeSequences(96)
BWHITE = ColorEscapeSequences(97)

BOLD = style.BOLD.escseq
DIM = style.DIM.escseq
ITALIC = style.ITALIC.escseq
UNDERLINE = style.UNDERLINE.escseq
DOUBLE_UNDERLINE = style.DOUBLE_UNDERLINE.escseq
BLINKING = style.BLINKING.escseq
RAPID_BLINKING = style.RAPID_BLINKING.escseq
INVERSE = style.INVERSE.escseq
HIDDEN = style.HIDDEN.escseq
STRIKETHROUGH = style.STRIKETHROUGH.escseq

RESET_ALL = ColorEscapeSequences.to_escseq(0)


def fg(ces: ColorEscapeSequences) -> str:
    return ces.fg


def bg(ces: ColorEscapeSequences) -> str:
    return ces.bg